﻿namespace TabsStudioPriorityGroup
{
    public class PriorityGroup : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler
    {
        public PriorityGroup()
        {
            addMenuItem = new System.Windows.Controls.MenuItem();
            addMenuItem.Header = "Add to priority group";
            addMenuItem.Click += OnAddToPriorityGroup;

            removeMenuItem = new System.Windows.Controls.MenuItem();
            removeMenuItem.Header = "Remove from priority group";
            removeMenuItem.Click += OnRemoveFromPriorityGroup;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            controller = new Controller(engine);
            engine.TabCreated += OnTabCreated;
            engine.TabDestroyed += OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
            engine.TabDestroyed -= OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioPriorityGroup=\"clr-namespace:TabsStudioPriorityGroup;assembly=PriorityGroup\"";
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            controller.OnTabCreated(e);
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            controller.OnTabDestroyed(e);
        }

        private void OpeningTabExtensionContextMenu(object sender, TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            PlaceMenuItem(e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            PlaceMenuItem(e.ContextMenu);
        }

        private void OnAddToPriorityGroup(object sender, System.Windows.RoutedEventArgs e)
        {
            controller.AddTabToPriorityGroup(storedTab);
        }

        private void OnRemoveFromPriorityGroup(object sender, System.Windows.RoutedEventArgs e)
        {
            controller.RemoveTabFromPriorityGroup(storedTab);
        }

        private void PlaceMenuItem(System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    if ((item as System.Windows.Controls.Control).Name.Equals("TabsStudioSeparator"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item), new System.Windows.Controls.Separator());
                        menu.Items.Insert(menu.Items.IndexOf(item), 
                            controller.IsPriorityTab(storedTab) ? removeMenuItem : addMenuItem);
                        break;
                    }
                }
            }
        }

        private System.Windows.Controls.MenuItem addMenuItem, removeMenuItem;
        private TabsStudioExt.ITabsStudioEngine engine;
        private Controller controller;
        private TabsStudioExt.Tab storedTab;
    }
}
